'use strict';
const express = require('express')
let request = require('request')
const process = require('process')

const dgram = require('dgram');
const { constants } = require('buffer');
// const { logger } = require('../../lib');
const server = dgram.createSocket('udp4');
const udpProxy = dgram.createSocket('udp4');

const app = express()
let port = 5000, resource_id, functionHash, runtime, idleTime = 60, flagFirstRequest = true
let waitTime

resource_id = process.argv[2]
functionHash = process.argv[3]
port = process.argv[4]
runtime = process.argv[5]
request = request.defaults({
    headers: { 'x-resource-id': resource_id }
});

let producer
try {
    let kafka = require('kafka-node'),
        Producer = kafka.Producer,
        client = new kafka.KafkaClient({
            kafkaHost: process.argv[6],
            autoConnect: true
        })
    producer = new Producer(client)
} catch(e) {
    console.log("Exception: ", e);
}


app.use(express.json());
app.use(express.urlencoded({ extended: true }));

let lastRequest = Date.now(), totalRequest = 0
app.post('/serverless/function/execute/', (req, res) => {
    if (flagFirstRequest) {
        waitTime = Date.now() - waitTime
        flagFirstRequest = false
    }
    let payload = req.body
    lastRequest = Date.now()
    totalRequest++
    executor(payload).then((result) => {
        res.json(result)
    })
})

app.post('/serverless/function/timeout', (req, res) => {
    console.log(req.body);
    
    let idleTime = req.body.timeout
    console.log("Idle time set to: ", idleTime);
    res.json({
        status: "success"
    })
})



async function executor(payload) {
    return new Promise((resolve, reject) => {
        
    })
}

app.listen(port, () => {
    console.log(`Resource ${resource_id} Server listening on port ${port}!`)
    if (producer)
        producer.send(
        [{
            topic: "deployed",
            messages: JSON.stringify({ functionHash, portExternal: port, 
                runtime, resource_id, entity_id: process.pid}),
            "status": true
        }], () => { })
    waitTime = Date.now()
})

function shouldDie() {
    
    if (Date.now() - lastRequest > idleTime * 1000) {
        let message = JSON.stringify({
            functionHash, portExternal: port,
            runtime, resource_id, entity_id: process.pid,
            total_request: totalRequest, wait_time: waitTime
        })

        console.log("Idle for too long. Exiting");
        if (producer)
            producer.send(
                [
                    {topic: "removeWorker", messages: message }
                ], (err, data) => {
                    if (err)
                        console.log(err);
                        
                    console.log("Ending worker for function", functionHash, "resource_id", resource_id);
                    process.exit(0)
                })
        
    }
}


server.on('error', (err) => {
    console.log(`server error:\n${err.stack}`);
    server.close();
});

server.on('message', (msg, rinfo) => {

    let payload = JSON.parse(msg)
    console.log(payload, typeof payload);

    executor(payload).then(result => {
        result = JSON.stringify(result)
        
        try {
            udpProxy.send(result, 0, result.length, "8080", "localhost", function (err, bytes) {
                if (err)
                    console.log(err)
                console.log("response via UDP")
            })
        } catch (e) {
            console.log(e)
        }

    })
});

server.on('listening', () => {
    const address = server.address();
    console.log(`server listening ${address.address}:${address.port}`);
});

server.bind(port);

setInterval(shouldDie, 1000);